// Hal for TI Launchpad using CCS debug console for display
#include <msp430g2553.h>
#include "pins.h"

/****************** Hardware Abstraction ********************/ 

/* modify these routines (and pin assignment in pins.h) for specific MCU

   halinit()            - init hardware (mcu, uart, spi, etc.)
   conout(char)         - output to debug console
   dir_in(pin)          - set pin direction to input
   dir_out(pin)         - set pin direction to output
   pin_high(pin)        - output pin = 1
   pin_low(pin)         - output pin = 0
   pin_in(pin)          - get input pin, returns 0 or 1
   wait_secs(secs)      - wait for secs seconds
   led_on               - turn led on
   led_off              - turn led off 
   shiftout(count,data) - SPI shiftout count bits from data
   shiftin(count)       - SPI shiftin count bits, return data
*/

unsigned char _CIOBUF_[12 + 64];// This *must* be global and named _CIOBUF_

/* init mcu-specific hardware */
#define clkrate 16  // set msp430 clock rate = 1, 4, 8 or 16 MHz

halinit(void) {
// mcu clock
	WDTCTL = WDTPW + WDTHOLD;    // Stop WDT
	DCOCTL = 0;                  // Set clock rate
#if clkrate == 1
	BCSCTL1 = CALBC1_1MHZ;
	DCOCTL = CALDCO_1MHZ;
#endif
#if clkrate == 4
	BCSCTL1 = CALBC1_4MHZ;
	DCOCTL = CALDCO_4MHZ;
#endif
#if clkrate == 8
	BCSCTL1 = CALBC1_8MHZ;
	DCOCTL = CALDCO_8MHZ;
#endif
#if clkrate == 16
	BCSCTL1 = CALBC1_16MHZ;
	DCOCTL = CALDCO_16MHZ;
#endif

// led
  dir_out(LED);               // enable led output
  led_off();                  // init off

//spi
/* - This is bit-bang spi implementation.  To use h/w spi
   - enable it here and rewrite shiftin & shiftout routines. */
  dir_in(MISO);               // enable miso,mosi,sclk
  pin_low(MOSI);
  dir_out(MOSI);
  pin_low(SCLK);
  dir_out(SCLK);
}  // end halinit

/******************   SPI Primitives   ***********************/
/*                Bit-bang SPI interface                     */ 
/*   shiftout and shiftin can be rewritten for hardware SPI  */
/*************************************************************/

shiftin(count){  // shiftin 1-16 bits MSB first
   int spidata = 0;
   pin_low(SCLK);
   pin_low(MOSI);
   int i;
   for (i=0; i < count; i++) {
      spidata = (spidata << 1) | pin_in(MISO);
      pin_high(SCLK);
      pin_low(SCLK);
   }
   return spidata;
}

shiftout(count, spidata){  // shiftout 1-16 bits MSB first
	pin_low(SCLK);
    spidata = spidata << (16 - count);
    int i;
    for (i=0; i < count; i++) {
      if((spidata & 0x8000) == 0x8000) pin_high(MOSI);
      else pin_low(MOSI);
      spidata = spidata << 1;
      pin_high(SCLK);
      pin_low(SCLK);
    }
}

/* output char to Code Composer Studio (CSS) debug console
http://www.43oh.com/forum/viewtopic.php?f=10&t=2953&sid=25d636fea773a75638f66c0c51c26378 */

conout(char out) {
	static char * p = (char *)&_CIOBUF_[11];    // Pointer to string buffer
	if(out) *p++ = out;                         // Append any non-null char to buffer
	if((p >= (char *)&_CIOBUF_[sizeof(_CIOBUF_)-1]) || (out==13)) {  // buffer full or cr
	   *p = 0;                                  // Null terminate string
	    const unsigned l = p - (char *)&_CIOBUF_[10];// String length including null
	    _CIOBUF_[0] = _CIOBUF_[5] = l & 0xFF;   // Data and string length LSB
	    _CIOBUF_[1] = _CIOBUF_[6] = l >> 8;     // Data and string length MSB
	    _CIOBUF_[2] = 0xF3;                     // Write command
	    _CIOBUF_[3] = 1; _CIOBUF_[4] = 0;       // stdout stream
	    __asm(" .global C$$IO$$");              // CIO breakpoint
	    __asm("C$$IO$$:nop");                   //
	    p = (char *)&_CIOBUF_[11];              // Reset string buffer pointer
	}                                           //
}

/* pin primitives */

dir_in(int pin) {
  if (pin < 8) P1DIR &= ~(1 << pin);  // set pin direction to input
  else P2DIR &= ~(1 << (pin - 8));
}

dir_out(int pin) {
  if (pin < 8) P1DIR |= 1 << pin;
  else P2DIR |= (1 << (pin - 8));     // set pin direction to output
}

pin_high(int pin) {
  if (pin < 8) P1OUT |= (1 << pin);  // output pin = 1
	  else P2OUT |= (1 << (pin - 8));
}

pin_low(int pin) {
  if (pin < 8) P1OUT &= ~(1 << pin);  // output pin = 0
  else P2OUT &= ~(1 << (pin - 8));
}

pin_in(int pin) {  // get input pin, return 0 or 1
  if (pin < 8) return (P1IN >> pin) & 0x1;
  else return (P2IN >> pin) & 0x1;
}

/* timing primitives */

wait_secs(int secs) {         // wait for 1 second
  do {
#if clkrate == 1
	__delay_cycles(1000000);
#endif
#if clkrate == 4
	__delay_cycles(4000000);
#endif
#if clkrate == 8
	__delay_cycles(8000000);
#endif
#if clkrate == 16
	__delay_cycles(16000000);
#endif
  } while (secs-- > 0);
}

/* led primitives */

led_on(void) {          // turn led on
  pin_high(LED);
}

led_off(void) {        // turn led off
  pin_low(LED);
}

/*
serout_h(out, count){      // out count hex digits
  if (count == 0) return;
  long int temp;
  do {
    temp = (out >> ((count-1) << 2)) & 0x0f;
    if (temp < 0x0a) serout(temp + '0');
    else serout((temp - 0x0a) + 'A');
    count--;
  } while (count > 0);
}
serout(out){      // out count hex digits
  conout(out);
}

	// random number generator
		  unsigned int i;
		  int result = 0;
		  // setup Timer_A
		  TA0CCTL0 = CM_1 + CCIS_1 + CAP;
		  TA0CTL |= TASSEL_2 + MC_2;
		  for(i=0 ; i<16 ; i++) {
		    // shift left result
		    result <<= 1;
		    // wait until Capture flag is set
		    while(!(TA0CCTL0 & CCIFG));
		    	// clear flag
		    TA0CCTL0 &= ~CCIFG;
		    // check LSB
		    if(TA0CCR0 & 0x01) result |= 0x01;
		    // change the divison of timer input clock
//		    TA0CTL = (TA0CTL & 0xFCFF) | ((TA0CCR0 & 0x03) << 8);
		  }

		  srand(result);
*/

