
/***********************************************************
*	Copyright 2011  C. Scott Weber
*	Arlington TX.  76018
*
*	Permissionis granted to use the code 
*	for other purposes provided appropiate credit is
*	given in any publish or sold materials
*	
*	No warrantee is given for the usefulenss or reliability 
*	of anything provided here.
*
*	CONTROLLER.C  This modules handles the 
*	controls (push buttons) that the user 
*	presses, as updates the text over each button
****************************************************/
#include "p18f2523.h"
#include <stdio.h>
#include <string.h>
#include <usart.h>
#include <eep.h>
#include <delays.h>

#include "extern.h"

#include "lcd.h"

#include "tools.h"
#include "view.h"
#include "controller.h"
#include "model.h"

/* The menu states */
typedef enum  {
	MENULVL0 = 0,
	MENULVL1,
	MENULVL2,
	MENULVL3,
	MENULVL4,
	MENULVL5,
	MENULVL6,
	MENULVL7,
	MENULVL8,
	MENULVL9,
	MENULVL10
} MENU_STATE;

/*  Some stuff to make testing for button presses easier */
#define BUTTONA (!(buttons & 0x04))
#define BUTTONB (!(buttons & 0x08))
#define BUTTONC (!(buttons & 0x10))


/* Make all text 6 chars wide, so they are sure to overwrite */
rom char MENU0[18][7] = { "Garage","Lights"," Time ","Status",
						"Acura ","Tesla "," Open ","Close ",
						"Porch ","Garage"," On   "," Off  ",
						" DST  "," x26  "," x27  ",	" x38  ",
						" x39  ",	" x50  "};

rom char SELECT[] =   "Select";
rom char STOP[] =		" STOP ";
rom char BACK[] =		" Back ";

/* The addresses of the devices */
#define TESLA			0x38
#define ACURA			0x39
#define PORCH			0x27
#define GARAGE		0x26
#define TIME			0x50

/* Locals to this module */
static unsigned char buttons, keypress;
static unsigned char state;
static unsigned char showStop;

/* An outgoing command message, in case we need to send 
	somethign to a device	*/
static unsigned char command[16];

/* Prototypes so the compiler knows these are to come */
void ScanKeys(void) ;
void ProcessKeys(void);
void ProcessState(void);
void DrawMenu(void);
void SendRequest(void);


void InitializeController(void) {
	buttons = PORTC & 0x1c;
	state = 0;
	/*  Pre-build the knowns for the command buffer 
	We will use the target address as a flag to indicate that 
	the message is ready to be sent. */
 	command[0] = 0x00;
	command[2] = ADDR;
	keypress = 0;
	showStop = 0;
}



void ProcessController(void) {

	ScanKeys();		/* Check for key presses */

	if ((sec == 0) && (state == 0)) {
		PrintTime();		/* If we are at the top, and the clock
							kicks over, update the time */
	}
	if (ALARM_ON) {		/* if the alarm is going */
		if (!showStop) {	/* if we're not showing 'STOP' */
			DrawMenu();		/* Draw the menu	*/
			showStop  =1;		/* Now we are (so don't do it anymore */
		}
	} else {					/* or of NOT in alarm */
		if (showStop) {	/*  If we were showing stop	*/
			showStop = 0;		/* now we won't	*/
			DrawMenu();		/* and draw a menu	*/
		}
	}


	/* If, at some point, 
		the SOH was set, then fire the message */
	if (command[0] == 0x01) {
		/*  We can block here. The send time at 19,200 is pretty quick
			for just 5 bytes ( 5 * 8 * 52 uS = 2.08 mS ) */
		SendRequest();
	}
}

		
void ScanKeys(void) {
	unsigned char temp;

	if (keypress) {		/* Was there a key press from the last time?  */
		keypress++;		/* count up (to debounce) */
		Delay10KTCYx(35);	/* Take a nap (slows done EVERYONE, but alas...) */
		if (keypress < 10) return ;	/* Wait for 10 of them	*/
	}				/*  Now the key press is valid	*/
	keypress = 0;			/* So clear the flag	*/
	temp = PORTC & 0x1c;		/* Only the center bits */

	if (temp == 0x1c) return;	/* If all button are up, then nothing happened */
	keypress = 1;				/* Otherwise, flag that we are in debounce */
	/* If the alarm is on, we turn it off, and don't even bother 
		activating the backlight  */
	if (ALARM_ON) {		/* Are we in an alarm ?	*/
		buttons = temp;		/*Get the button press */
		if (BUTTONC) {		/* And if its 'C', kill the alarm */
			ActivateAlarm(0);
			return;		/* and bail (the keypress will be discarded */
		}
	}

	if (LATCbits.LATC5 == 0) {		/* If the screen is not on...	*/
		ScreenOn();			/* Now it is...		*/
		return ;			/* and bail (discard the keypress ) */
	}
	buttons = temp;		/* Get the actual keypress */
	ScreenOn();			/* reinforce the screen being on */
	ProcessKeys(); 		/* And figure out what they want */
}

void ProcessKeys(void) {
	if (ALARM_ON) {		/* If the alarm was on */
		if (BUTTONC) {	/* Kill it	*/
			ActivateAlarm(0);
			return;
		}
	}
	ProcessState();	/* Other wise, process the state machine */
}

/*  This will advacne or revert to the state, based on what the previous 
	state was.  Consult the state diagram for the behavior */
void ProcessState(void) {
	switch (state) {
	case 0 :	if (BUTTONA) { state = 1; break; }
			if (BUTTONB) { state = 4; break; }	
					/*  Revert to STATUS on the view */
			if (BUTTONC) { SetMode(STATUS_DISP, 0); break; }
	case 1 :	if (BUTTONA) { state = 2; break; }
			if (BUTTONB) { state = 8; break; }	
	case 2 :	if (BUTTONA) { state = 3; break; }	
			if (BUTTONB) { state = 12; break; }	
	case 3 :	if (BUTTONA) { state = 0; break; }	
			if (BUTTONB) { state = 13; 
				command[3] = 'Z';	
				command[4] = 0x04;
				break; 
			}	
	case 4 :	if (BUTTONA) { state = 5; break; }	
			if (BUTTONB) { state = 6; command[1] = ACURA; break; }
			if (BUTTONC)  state = 0; 
			break;
	case 5 :	if (BUTTONA) { state = 4; break; }	
			if (BUTTONB) { state = 6; command[1] = TESLA; break; }
			if (BUTTONC)  state = 0; 
			break;
	case 6 :	if (BUTTONA) { state = 7; break; }
			if (BUTTONB) { 
				state = 0; 	/* Build a message */
				command[0] = 0x01;
				command[3] = 'O';	
				command[4] = 0x04;
				break; 
				}
			if (BUTTONC)  state = 0; 
			break;
	case 7 :	if (BUTTONA) { state = 6; break; }
			if (BUTTONB) { state = 0; 
				command[0] = 0x01;		/* Start to build a message */
				command[3] = 'C';	
				command[4] = 0x04;
				break; 
				}
			if (BUTTONC)  state = 0; 
			break;
	case 8 :	if (BUTTONA) { state = 9; break; }	
			/* Continue to build a message */
			if (BUTTONB) { state = 10; command[1] = PORCH; break; }
			if (BUTTONC)  state = 0; 
			break;
	case 9 :	if (BUTTONA) { state = 8; break; }	
			/* Continue to build a message */
			if (BUTTONB) { state = 10; command[1] = GARAGE; break; }
			if (BUTTONC)  state = 0; 
			break;
	case 10 :	if (BUTTONA) { state = 11; break; }
			if (BUTTONB) { state = 0; 
				command[0] = 0x01;		/* Now the msg is complete */
				command[3] = 'L';	
				command[4] = '1';	
				command[5] = 0x04;
				break; 
				}
			if (BUTTONC)  state = 0; 
			break;
	case 11 :if (BUTTONA) { state = 10; break; }
			if (BUTTONB) { state = 0; 
				command[0] = 0x01;		/* Now the msg is complete */
				command[3] = 'L';	
				command[4] = '0';	
				command[5] = 0x04;
				break; 
				}
			if (BUTTONC)  state = 0; 
			break;
	case 12 :if (BUTTONB) { state = 0; 
				command[0] = 0x01;  /* Build a whole msg */
				command[1] = TIME; 
				command[3] = 'D';	/* Set the DST based on the current GMT */
				if (client[4].buffer[0] == 6) command[4] = 0x25;
				else  command[4] = 0x26;
				command[5] = 0x04;
				break; 
				}
			if (BUTTONC)	state = 0; 
			break;
	case 13 :if (BUTTONA) { state = 14; break; }
			if (BUTTONB) { state = 0; 
				command[0] = 0x01;
				command[1] = PORCH; 
				SetMode(STATUS_RST, LIGHT1);
				break; 
			}	
			if (BUTTONC) 	state = 0;
			break;
	case 14 :if (BUTTONA) { state = 15; break; }
			if (BUTTONB) { state = 0; 
				command[0] = 0x01;
				command[1] = GARAGE; 
				SetMode(STATUS_RST, LIGHT2);
				break; 
			}	
			if (BUTTONC) state = 0; 
			break;
	case 15 :if (BUTTONA) { state = 16; break; }
			if (BUTTONB) { state = 0; 
				command[0] = 0x01;		/* Now the msg is complete */
				command[1] = ACURA; 
				SetMode(STATUS_RST, GARAGE1);
				break; 
			}	
			if (BUTTONC) 	state = 0; 
			break; 
	case 16 :if (BUTTONA) { state = 17; break; }
			if (BUTTONB) { state = 0; 
				command[0] = 0x01;		/* Now the msg is complete */
				command[1] = TESLA; 
				SetMode(STATUS_RST, GARAGE1);
				break; 
			}	
			if (BUTTONC) state = 0; 
			break;
	case 17 :if (BUTTONA) { state = 13; break; }
			if (BUTTONB) { state = 0; 
				command[0] = 0x01;		/* Now the msg is complete */
				command[1] = TIME; 
				SetMode(STATUS_RST, 4);
				break; 
			}
			if (BUTTONC)  state = 0; 
			break;	
	}
	/* Update the menu based on the new state  */
	DrawMenu();
}

void DrawMenu(void) {

	WriteRom(SELECT,BUTTON1X);	/* Draw button 1 ('select') */
		/* Draw button 0 based on the current state */
	WriteRom(MENU0[state],BUTTON0X); 

	/* And draw button 3 based on the state, alarm, or just need the time */
	if (ALARM_ON) {
		WriteRom(STOP, BUTTON2X);
		return;
	}
	if (state < 4) {
		CursorJump(BUTTON2X);
		PrintTime();
	} else {
		WriteRom(BACK,BUTTON2X);
	}

}


void SendRequest(void) {
	unsigned char index;
	/* If we are in the process of receiving, then bail for now.
		The next pass through the controller wil catch it */
	if (cmd_state != CMD_WAITING) {
		return;
	}
	LATAbits.LATA2 = 1;	/*  Enable the RS485 line	*/
	index = 0;
	do {
		while (BusyUSART())  { ;  }
		WriteUSART(command[index]);
		if (command[index] == 0x04) break;
		index++;
	} while (1);
		
	while (BusyUSART())  { ;  }	/* Wait for it to complete */			
	LATAbits.LATA2 = 0;			/*  Disable the RS485 line  */
	command[0] = 0x00;
}



